@@ -263,7 +263,7 @@ func backupTarget(bucket Bucket, file string, keepOriginal bool) string {
 	contract.Require(file != "", "file")
 	bck := file + ".bak"
 
-	err := bucket.Copy(context.TODO(), file, bck, nil)
+	err := bucket.Copy(context.TODO(), bck, file, nil)
 	if err != nil {
 		logging.V(5).Infof("error copying %s to %s: %s", file, bck, err)
 	}
