@@ -5,6 +5,8 @@ import (
 	"fmt"
 	gofmt "go/format"
 	"io"
+	"io/ioutil"
+	"path"
 	"strings"
 	"sync"
 
@@ -17,7 +19,9 @@ import (
 	"github.com/pulumi/pulumi/pkg/v3/codegen/hcl2/syntax"
 	"github.com/pulumi/pulumi/pkg/v3/codegen/pcl"
 	"github.com/pulumi/pulumi/pkg/v3/codegen/schema"
+	"github.com/pulumi/pulumi/sdk/v3/go/common/encoding"
 	"github.com/pulumi/pulumi/sdk/v3/go/common/util/contract"
+	"github.com/pulumi/pulumi/sdk/v3/go/common/workspace"
 )
 
 type generator struct {
@@ -119,6 +123,68 @@ func GenerateProgramWithOptions(program *pcl.Program, opts GenerateProgramOption
 	return files, g.diagnostics, nil
 }
 
+func GenerateProject(directory string, project workspace.Project, program *pcl.Program) error {
+	files, diagnostics, err := GenerateProgram(program)
+	if err != nil {
+		return err
+	}
+	if diagnostics.HasErrors() {
+		return diagnostics
+	}
+
+	// Set the runtime to "go" then marshal to Pulumi.yaml
+	project.Runtime = workspace.NewProjectRuntimeInfo("go", nil)
+	projectBytes, err := encoding.YAML.Marshal(project)
+	if err != nil {
+		return err
+	}
+	files["Pulumi.yaml"] = projectBytes
+
+	// Build a go.mod based on the packages used by program
+	var gomod bytes.Buffer
+	gomod.WriteString("module " + project.Name.String() + "\n")
+	gomod.WriteString(`
+go 1.17
+
+require (
+	github.com/pulumi/pulumi/sdk/v3 v3.30.0
+`)
+
+	// For each package add a PackageReference line
+	packages := program.Packages()
+	for _, p := range packages {
+		if err := p.ImportLanguages(map[string]schema.Language{"go": Importer}); err != nil {
+			return err
+		}
+
+		goInfo := p.Language["go"].(GoPackageInfo)
+		vPath := ""
+		if p.Version != nil && p.Version.Major > 1 {
+			vPath = fmt.Sprintf("/v%d", p.Version.Major)
+		}
+		packageName := fmt.Sprintf("github.com/pulumi/pulumi-%s/sdk%s/go/%s", p.Name, vPath, p.Name)
+		if goInfo.ImportBasePath != "" {
+			packageName = goInfo.ImportBasePath
+		}
+
+		gomod.WriteString(fmt.Sprintf("	%s v%s\n", packageName, p.Version.String()))
+	}
+
+	gomod.WriteString(")")
+
+	files["go.mod"] = gomod.Bytes()
+
+	for filename, data := range files {
+		outPath := path.Join(directory, filename)
+		err := ioutil.WriteFile(outPath, data, 0600)
+		if err != nil {
+			return fmt.Errorf("could not write output program: %w", err)
+		}
+	}
+
+	return nil
+}
+
 var packageContexts sync.Map
 
 func getPackages(tool string, pkg *schema.Package) map[string]*pkgContext {
@@ -474,7 +540,7 @@ func (g *generator) genResourceOptions(w io.Writer, block *model.Block) {
 
 func (g *generator) genResource(w io.Writer, r *pcl.Resource) {
 
-	resName, resNameVar := r.Name(), makeValidIdentifier(r.Name())
+	resName, resNameVar := r.LogicalName(), makeValidIdentifier(r.Name())
 	pkg, mod, typ, _ := r.DecomposeToken()
 	if mod == "" || strings.HasPrefix(mod, "/") || strings.HasPrefix(mod, "index/") {
 		mod = pkg
@@ -562,7 +628,7 @@ func (g *generator) genResource(w io.Writer, r *pcl.Resource) {
 func (g *generator) genOutputAssignment(w io.Writer, v *pcl.OutputVariable) {
 	expr, temps := g.lowerExpression(v.Value, v.Type())
 	g.genTemps(w, temps)
-	g.Fgenf(w, "ctx.Export(\"%s\", %.3v)\n", v.Name(), expr)
+	g.Fgenf(w, "ctx.Export(%q, %.3v)\n", v.LogicalName(), expr)
 }
 func (g *generator) genTemps(w io.Writer, temps []interface{}) {
 	singleReturn := ""
@@ -673,7 +739,9 @@ func (g *generator) genLocalVariable(w io.Writer, v *pcl.LocalVariable) {
 				g.Fgenf(w, "return err\n")
 				g.Fgenf(w, "}\n")
 			}
-		case "join", "toBase64", "mimeType", "fileAsset":
+		case "join", "toBase64", "mimeType",
+			"fileArchive", "remoteArchive", "assetArchive",
+			"fileAsset", "stringAsset", "remoteAsset":
 			g.Fgenf(w, "%s := %.3v;\n", name, expr)
 		}
 	default:
