@@ -1,10 +1,12 @@
 package gen
 
 import (
+	"bytes"
 	"encoding/json"
 	"fmt"
 	"io/ioutil"
 	"path/filepath"
+	"regexp"
 	"sort"
 	"strings"
 	"testing"
@@ -360,3 +362,44 @@ func importSpec(t *testing.T, spec schema.PackageSpec) *schema.Package {
 	assert.NoError(t, err)
 	return importedPkg
 }
+
+func TestGenHeader(t *testing.T) {
+	t.Parallel()
+
+	pkg := &pkgContext{
+		tool: "a tool",
+		pkg:  &schema.Package{Name: "test-pkg"},
+	}
+
+	s := func() string {
+		b := &bytes.Buffer{}
+		pkg.genHeader(b, []string{"pkg1", "example.com/foo/123-foo"}, nil)
+		return b.String()
+	}()
+	assert.Equal(t, `// Code generated by a tool DO NOT EDIT.
+// *** WARNING: Do not edit by hand unless you're certain you know what you are doing! ***
+
+package testpkg
+
+import (
+	"pkg1"
+	"example.com/foo/123-foo"
+)
+
+`, s)
+
+	// Compliance is defined by https://pkg.go.dev/cmd/go#hdr-Generate_Go_files_by_processing_source
+	autogenerated := regexp.MustCompile(`^// Code generated .* DO NOT EDIT\.$`)
+	found := false
+	for _, l := range strings.Split(s, "\n") {
+		switch {
+		case autogenerated.Match([]byte(l)):
+			found = true
+			break
+		case l == "" || strings.HasPrefix(l, "//"):
+		default:
+			break
+		}
+	}
+	assert.Truef(t, found, `Didn't find a line that complies with "%v"`, autogenerated)
+}
