@@ -18,6 +18,7 @@ import (
 	"bytes"
 	"fmt"
 	"io"
+	"io/ioutil"
 	"path"
 	"sort"
 	"strings"
@@ -29,7 +30,9 @@ import (
 	"github.com/pulumi/pulumi/pkg/v3/codegen/hcl2/syntax"
 	"github.com/pulumi/pulumi/pkg/v3/codegen/pcl"
 	"github.com/pulumi/pulumi/pkg/v3/codegen/schema"
+	"github.com/pulumi/pulumi/sdk/v3/go/common/encoding"
 	"github.com/pulumi/pulumi/sdk/v3/go/common/util/contract"
+	"github.com/pulumi/pulumi/sdk/v3/go/common/workspace"
 	"github.com/zclconf/go-cty/cty"
 )
 
@@ -65,10 +68,19 @@ func GenerateProgram(program *pcl.Program) (map[string][]byte, hcl.Diagnostics,
 	var index bytes.Buffer
 	g.genPreamble(&index, program, preambleHelperMethods)
 	for _, n := range nodes {
-		if r, ok := n.(*pcl.Resource); ok && requiresAsyncMain(r) {
-			g.asyncMain = true
+		if g.asyncMain {
 			break
 		}
+		switch x := n.(type) {
+		case *pcl.Resource:
+			if resourceRequiresAsyncMain(x) {
+				g.asyncMain = true
+			}
+		case *pcl.OutputVariable:
+			if outputRequiresAsyncMain(x) {
+				g.asyncMain = true
+			}
+		}
 	}
 
 	indenter := func(f func()) { f() }
@@ -89,14 +101,15 @@ func GenerateProgram(program *pcl.Program) (map[string][]byte, hcl.Diagnostics,
 					if result == nil {
 						result = &model.ObjectConsExpression{}
 					}
-					name := makeValidIdentifier(o.Name())
+					name := o.LogicalName()
+					nameVar := makeValidIdentifier(o.Name())
 					result.Items = append(result.Items, model.ObjectConsItem{
 						Key: &model.LiteralValueExpression{Value: cty.StringVal(name)},
 						Value: &model.ScopeTraversalExpression{
-							RootName:  name,
+							RootName:  nameVar,
 							Traversal: hcl.Traversal{hcl.TraverseRoot{Name: name}},
 							Parts: []model.Traversable{&model.Variable{
-								Name:         name,
+								Name:         nameVar,
 								VariableType: o.Type(),
 							}},
 						},
@@ -120,6 +133,97 @@ func GenerateProgram(program *pcl.Program) (map[string][]byte, hcl.Diagnostics,
 	return files, g.diagnostics, nil
 }
 
+func GenerateProject(directory string, project workspace.Project, program *pcl.Program) error {
+	files, diagnostics, err := GenerateProgram(program)
+	if err != nil {
+		return err
+	}
+	if diagnostics.HasErrors() {
+		return diagnostics
+	}
+
+	// Set the runtime to "nodejs" then marshal to Pulumi.yaml
+	project.Runtime = workspace.NewProjectRuntimeInfo("nodejs", nil)
+	projectBytes, err := encoding.YAML.Marshal(project)
+	if err != nil {
+		return err
+	}
+	files["Pulumi.yaml"] = projectBytes
+
+	// Build the pacakge.json
+	var packageJSON bytes.Buffer
+	packageJSON.WriteString(fmt.Sprintf(`{
+		"name": "%s",
+		"devDependencies": {
+			"@types/node": "^14"
+		},
+		"dependencies": {
+			"@pulumi/pulumi": "^3.0.0"`, project.Name.String()))
+	// For each package add a dependency line
+	packages := program.Packages()
+	for _, p := range packages {
+		if err := p.ImportLanguages(map[string]schema.Language{"go": Importer}); err != nil {
+			return err
+		}
+
+		info := p.Language["nodejs"].(NodePackageInfo)
+		packageName := "@pulumi/" + p.Name
+		if info.PackageName != "" {
+			packageName = info.PackageName
+		}
+		dependencyTemplate := ",\n			\"%s\": \"%s\""
+		packageJSON.WriteString(fmt.Sprintf(dependencyTemplate, packageName, p.Version.String()))
+	}
+	packageJSON.WriteString(`
+		}
+}`)
+
+	files["package.json"] = packageJSON.Bytes()
+
+	// Add the language specific .gitignore
+	files[".gitignore"] = []byte(`/bin/
+/node_modules/`)
+
+	// Add the basic tsconfig
+	var tsConfig bytes.Buffer
+	tsConfig.WriteString(`{
+		"compilerOptions": {
+			"strict": true,
+			"outDir": "bin",
+			"target": "es2016",
+			"module": "commonjs",
+			"moduleResolution": "node",
+			"sourceMap": true,
+			"experimentalDecorators": true,
+			"pretty": true,
+			"noFallthroughCasesInSwitch": true,
+			"noImplicitReturns": true,
+			"forceConsistentCasingInFileNames": true
+		},
+		"files": [
+`)
+
+	for file := range files {
+		if strings.HasSuffix(file, ".ts") {
+			tsConfig.WriteString("			\"" + file + "\"\n")
+		}
+	}
+
+	tsConfig.WriteString(`		]
+}`)
+	files["tsconfig.json"] = tsConfig.Bytes()
+
+	for filename, data := range files {
+		outPath := path.Join(directory, filename)
+		err := ioutil.WriteFile(outPath, data, 0600)
+		if err != nil {
+			return fmt.Errorf("could not write output program: %w", err)
+		}
+	}
+
+	return nil
+}
+
 // genLeadingTrivia generates the list of leading trivia assicated with a given token.
 func (g *generator) genLeadingTrivia(w io.Writer, token syntax.Token) {
 	// TODO(pdg): whitespace?
@@ -226,7 +330,7 @@ func (g *generator) genNode(w io.Writer, n pcl.Node) {
 	}
 }
 
-func requiresAsyncMain(r *pcl.Resource) bool {
+func resourceRequiresAsyncMain(r *pcl.Resource) bool {
 	if r.Options == nil || r.Options.Range == nil {
 		return false
 	}
@@ -234,6 +338,15 @@ func requiresAsyncMain(r *pcl.Resource) bool {
 	return model.ContainsPromises(r.Options.Range.Type())
 }
 
+func outputRequiresAsyncMain(ov *pcl.OutputVariable) bool {
+	outputName := ov.LogicalName()
+	if makeValidIdentifier(outputName) != outputName {
+		return true
+	}
+
+	return false
+}
+
 // resourceTypeName computes the NodeJS package, module, and type name for the given resource.
 func resourceTypeName(r *pcl.Resource) (string, string, string, hcl.Diagnostics) {
 	// Compute the resource type from the Pulumi type token.
@@ -242,9 +355,16 @@ func resourceTypeName(r *pcl.Resource) (string, string, string, hcl.Diagnostics)
 		pkg, module, member = member, "", "Provider"
 	}
 
-	// Normalize module.
 	if r.Schema != nil {
-		pkg := r.Schema.Package
+		module = moduleName(module, r.Schema.Package)
+	}
+
+	return makeValidIdentifier(pkg), module, title(member), diagnostics
+}
+
+func moduleName(module string, pkg *schema.Package) string {
+	// Normalize module.
+	if pkg != nil {
 		if lang, ok := pkg.Language["nodejs"]; ok {
 			pkgInfo := lang.(NodePackageInfo)
 			if m, ok := pkgInfo.ModuleToPackage[module]; ok {
@@ -252,9 +372,7 @@ func resourceTypeName(r *pcl.Resource) (string, string, string, hcl.Diagnostics)
 			}
 		}
 	}
-
-	module = strings.ToLower(strings.Replace(module, "/", ".", -1))
-	return makeValidIdentifier(pkg), module, title(member), diagnostics
+	return strings.ToLower(strings.ReplaceAll(module, "/", "."))
 }
 
 // makeResourceName returns the expression that should be emitted for a resource's "name" parameter given its base name
@@ -307,7 +425,7 @@ func (g *generator) genResourceOptions(opts *pcl.ResourceOptions) string {
 	}
 
 	var buffer bytes.Buffer
-	g.Fgenf(&buffer, ", %v", g.lowerExpression(object))
+	g.Fgenf(&buffer, ", %v", g.lowerExpression(object, nil))
 	return buffer.String()
 }
 
@@ -324,8 +442,8 @@ func (g *generator) genResource(w io.Writer, r *pcl.Resource) {
 
 	optionsBag := g.genResourceOptions(r.Options)
 
-	name := r.Name()
-	variableName := makeValidIdentifier(name)
+	name := r.LogicalName()
+	variableName := makeValidIdentifier(r.Name())
 
 	g.genTrivia(w, r.Definition.Tokens.GetType(""))
 	for _, l := range r.Definition.Tokens.GetLabels(nil) {
@@ -340,17 +458,21 @@ func (g *generator) genResource(w io.Writer, r *pcl.Resource) {
 			indenter = g.Indented
 		}
 		indenter(func() {
+			fmtString := "%s: %.v"
+			if len(r.Inputs) > 1 {
+				fmtString = "\n" + g.Indent + "%s: %.v,"
+			}
+
 			for _, attr := range r.Inputs {
 				propertyName := attr.Name
 				if !isLegalIdentifier(propertyName) {
 					propertyName = fmt.Sprintf("%q", propertyName)
 				}
 
-				if len(r.Inputs) == 1 {
-					g.Fgenf(w, "%s: %.v", propertyName, g.lowerExpression(attr.Value))
-				} else {
-					g.Fgenf(w, "\n%s%s: %.v,", g.Indent, propertyName, g.lowerExpression(attr.Value))
-				}
+				destType, diagnostics := r.InputType.Traverse(hcl.TraverseAttr{Name: attr.Name})
+				g.diagnostics = append(g.diagnostics, diagnostics...)
+				g.Fgenf(w, fmtString, propertyName,
+					g.lowerExpression(attr.Value, destType.(model.Type)))
 			}
 		})
 		if len(r.Inputs) > 1 {
@@ -361,7 +483,7 @@ func (g *generator) genResource(w io.Writer, r *pcl.Resource) {
 
 	if r.Options != nil && r.Options.Range != nil {
 		rangeType := model.ResolveOutputs(r.Options.Range.Type())
-		rangeExpr := g.lowerExpression(r.Options.Range)
+		rangeExpr := g.lowerExpression(r.Options.Range, rangeType)
 
 		if model.InputType(model.BoolType).ConversionFrom(rangeType) == model.SafeConversion {
 			g.Fgenf(w, "%slet %s: %s | undefined;\n", g.Indent, variableName, qualifiedMemberName)
@@ -429,14 +551,14 @@ func (g *generator) genConfigVariable(w io.Writer, v *pcl.ConfigVariable) {
 
 	g.Fgenf(w, "%[1]sconst %[2]s = config.%[3]s%[4]s(\"%[2]s\")", g.Indent, v.Name(), getOrRequire, getType)
 	if v.DefaultValue != nil {
-		g.Fgenf(w, " || %.v", g.lowerExpression(v.DefaultValue))
+		g.Fgenf(w, " || %.v", g.lowerExpression(v.DefaultValue, v.DefaultValue.Type()))
 	}
 	g.Fgenf(w, ";\n")
 }
 
 func (g *generator) genLocalVariable(w io.Writer, v *pcl.LocalVariable) {
 	// TODO(pdg): trivia
-	g.Fgenf(w, "%sconst %s = %.3v;\n", g.Indent, v.Name(), g.lowerExpression(v.Definition.Value))
+	g.Fgenf(w, "%sconst %s = %.3v;\n", g.Indent, v.Name(), g.lowerExpression(v.Definition.Value, v.Type()))
 }
 
 func (g *generator) genOutputVariable(w io.Writer, v *pcl.OutputVariable) {
@@ -445,7 +567,8 @@ func (g *generator) genOutputVariable(w io.Writer, v *pcl.OutputVariable) {
 	if g.asyncMain {
 		export = ""
 	}
-	g.Fgenf(w, "%s%sconst %s = %.3v;\n", g.Indent, export, makeValidIdentifier(v.Name()), g.lowerExpression(v.Value))
+	g.Fgenf(w, "%s%sconst %s = %.3v;\n", g.Indent, export,
+		makeValidIdentifier(v.Name()), g.lowerExpression(v.Value, v.Type()))
 }
 
 func (g *generator) genNYI(w io.Writer, reason string, vs ...interface{}) {
